/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 * Copyright(c) 2005, MediaTek, Inc.
 * All rights reserved.
 *
 * Unauthorized use, practice, perform, copy, distribution, reproduction,
 * or disclosure of this information in whole or in part is prohibited.
 *-----------------------------------------------------------------------------
 * $Author: xiaojun.zhu $
 * $Date: 2015/08/08 $
 * $RCSfile: vdec_vp6.h,v $
 * $Revision: #3 $
 *---------------------------------------------------------------------------*/

/** @file vdec_vp8webp.h
 *  This header file declares exported APIs of MPV.
 */

#ifndef VDEC_VP8_WEBP_H
#define VDEC_VP8_WEBP_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_lint.h"
LINT_EXT_HEADER_BEGIN
#include "x_os.h"
#include "x_assert.h"
#include "x_typedef.h"
//#include "x_debug.h"
LINT_EXT_HEADER_END
;
#define WEBP_DEC0 (0)
#define WEBP_DEC1 (1)

//#define CC_WEBP_EMULATION

#if defined(__MODEL_slt__) || defined(CC_WEBP_EMULATION)
    #define WEBP_ENABLE_CRC
#endif

#define WEBP_CLRFLG(flg,index) ((flg) &= ~(1<<(index)))
#define WEBP_SETFLG(flg,index) ((flg) |= (1<<(index)))
#define WEBP_FLGSET(flg,index) (((flg)>>(index))&1)

typedef enum
{
   WEBP_NOTIFY_TYPE_REQDATA,
   WEBP_NOTIFY_TYPE_ROWNOTIFY,
}WEBP_NOTIFY_TYPE;

typedef enum
{
  WEBP_DEC_FLAG_KEEPRATIO,
  WEBP_DEC_FLAG_MAX
}WEBP_DEC_FLAG;

typedef enum
{
  WEBP_DEC_MODE_MCUROW,
  WEBP_DEC_MODE_PIC,
  WEBP_DEC_MODE_AUTO
}WEBP_DEC_MODE;

typedef enum
{
  WEBP_COLOR_AYCBCR8888,
  WEBP_COLOR_ARGB8888,
  WEBP_COLOR_YCSEP420,
  WEBP_COLOR_YCSEP422
}WEBP_COLOR_MODE;

typedef struct
{
  UINT32 u4FileOffset;
  UINT32 u4Size;
  UINT32 u4WritePosition;
}WEBP_DATA_REQ_INFO;

typedef struct
{
  UINT32 u4Pitch;
  UINT32 u4Height;
  UINT32 u4YAddr;
  UINT32 u4CAddr;
}WEBP_RZROW_INFO;

typedef struct
{
	UPTR u4DestBuffer;		// Target buffer
	UINT32 u4CbCrOffset;        // address offset for Y/CbCr seperate mode
	UINT32 u4Pitch;				// target buffer pitch
	UINT32 u4StartX;			//start point X
	UINT32 u4StartY;		    //start point Y
	UINT32 u4Width;				//region width
	UINT32 u4Height;            //region height
	UINT32 u4FileOffset;
	UINT32 u4FileSize;
	UINT32 u4Tag;
	UINT32 u4NotifyInfo;
	UINT32 u4Flag;
    WEBP_COLOR_MODE eColorMode;
#ifdef CC_WEBP_EMULATION
    UINT32 u4DecOutY;
    UINT32 u4DecOutC;
#endif
}WEBP_DECIN_PARAM_T;

typedef struct
{
	UINT32 u4CbCrOffset;        // address offset for Y/CbCr seperate mode
	UINT32 u4StartX;			//start point X
	UINT32 u4StartY;		    //start point Y
	UINT32 u4Width;				//region width
	UINT32 u4Height;            //region height
	UINT32 u4SrcWidht;
    UINT32 u4SrcHeight;
    WEBP_DEC_MODE eDecMode;
#ifdef WEBP_ENABLE_CRC
    UINT32 u4PPType;
	UINT32 u4Ycrc[4];
    UINT32 u4Ccrc[4];
    UINT32 u4RzCrc;
#endif
}WEBP_DECOUT_PARAM_T;

typedef UINT32 (*PFN_EWBP_EVENT_NOTIFY)(
	UINT32 u4Tag,
	VOID *pv_data,
	WEBP_NOTIFY_TYPE eType
);

extern BOOL _VDEC_Vp8WebPInit(UINT32 u4DecId,PFN_EWBP_EVENT_NOTIFY pfnCb,
	UINT32 u4BufStart,UINT32 u4BufSize);

extern BOOL _VDEC_Vp8WebPDecStart(UINT32 u4DecId,WEBP_DEC_MODE eDecMode,
    WEBP_DECIN_PARAM_T *prDecInParam,WEBP_DECOUT_PARAM_T *prDecOutParam);

extern BOOL _VDEC_Vp8WebPDecStop(UINT32 u4DecId);
extern UINT32 _VDEC_Vp8WebPGetProgress(UINT32 u4DecId);

#endif  // VDEC_VP8_API_H

