/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: mtk_edid.h
 *
 * Project:
 * --------
 *   MT5365
 *
 * Description:
 * ------------
 *   Edid customization
 *
 * Author:
 * -------
 *
 *
 * Unauthorized use, practice, perform, copy, distribution, reproduction,
 * or disclosure of this information in whole or in part is prohibited.
 *-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: mtk_vga.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/
#ifndef _EDID_CUSTOMIZATION_H_
#define _EDID_CUSTOMIZATION_H_
#include "eeprom_if.h"
#include "panel.h"
#include "drvcust_if.h"
#include "drv_edid.h"

#ifndef CC_MTK_LOADER
#ifdef __MW_CLI_DEF__
#include "x_mid_cli.h"
#else
#include "x_drv_cli.h"
#endif
#include "x_os.h"
#include "x_stl_lib.h"

#include "drv_hdmi.h"
#endif


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------
typedef enum {
    PLF_OPT_SEGMENT_KM = 0,
    PLF_OPT_SEGMENT_KMW,
    PLF_OPT_SEGMENT_KML,
    PLF_OPT_SEGMENT_KL,
    PLF_OPT_SEGMENT_KC,
    PLF_OPT_SEGMENT_KLW,
    PLF_OPT_SEGMENT_KMLW,
    PLF_OPT_SEGMENT_CP,
    PLF_OPT_SEGMENT_CPL,
    PLF_OPT_SEGMENT_KC2,
    PLF_OPT_SEGMENT_LAST_VALID_ENTRY,
} PLF_OPT_SEGMENT_T;

typedef enum
{
    PANEL_EDID_WXGA_NON_XVYCC = 1,
    PANEL_EDID_FHD_NON_XVYCC,
    PANEL_EDID_FHD_XVYCC,
    PANEL_EDID_QFHD_NON_XVYCC,
    PANEL_EDID_QFHD_XVYCC
}HDMI_PANEL_EDID_TYPE;

typedef enum
{
    HDMI_EDID_WXGA_NON_XVYCC = 0,
    HDMI_EDID_WXGA_NON_XVYCC_ATMOS,
    HDMI_EDID_WXGA_NON_XVYCC_HDR,
    HDMI_EDID_WXGA_NON_XVYCC_HDR_ATMOS,
    HDMI_EDID_FHD_NON_XVYCC,
    HDMI_EDID_FHD_NON_XVYCC_ATMOS,
    HDMI_EDID_FHD_NON_XVYCC_HDR,
    HDMI_EDID_FHD_NON_XVYCC_HDR_ATMOS,
    HDMI_EDID_FHD_XVYCC,
    HDMI_EDID_FHD_XVYCC_ATMOS,
    HDMI_EDID_FHD_XVYCC_HDR,
    HDMI_EDID_FHD_XVYCC_HDR_ATMOS,
}HDMI_EDID_FLASH_IDX_TYPE;

typedef enum
{
    HDMI_EDID_FORMAT_TYPE_STANDARD,
    HDMI_EDID_FORMAT_TYPE_ENHANCE,
} HDMI_EDID_FORMAT_TYPE_T;

//---------------------------------------------------------------------------
// Imported functions
//---------------------------------------------------------------------------
extern void vHDMI_INTERNAL_EDID_Init(void);


//---------------------------------------------------------------------------
// Static function forward declarations
//---------------------------------------------------------------------------
#ifndef CC_MTK_LOADER
static INT32 _SifHDMISetEdidDdpCmd(INT32 i4Argc, const CHAR **szArgv);
static INT32 _SifHDMIQuerySonyEdidCmd(INT32 i4Argc, const CHAR **szArgv);
#endif

//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------
#ifndef CC_MTK_LOADER
// Table for HDMI EDID/HDCP cust command entry
CLI_EXEC_T arSifHDMIcustCmdTbl[] = {
    {"sddp", "sddp", _SifHDMISetEdidDdpCmd, NULL, "Set DDP ATMOS", CLI_GUEST},
    {"q", "q", _SifHDMIQuerySonyEdidCmd, NULL, "Query Sony Edid information", CLI_GUEST},
    {NULL, NULL, NULL, NULL, NULL, CLI_SUPERVISOR}
};

static CHAR *_aszSonySegment[] =
{
    "KM",
    "KMW",
    "KML",
    "KL",
    "KC",
    "KL",
    "KMLW",
    "CP",
    "CPL",
    "KC2",
};

static CHAR *_aszSonyEdidName[] =
{
    "WXGA_NON_XVYCC",
    "WXGA_NON_XVYCC_ATMOS",
    "WXGA_NON_XVYCC_HDR",
    "WXGA_NON_XVYCC_HDR_ATMOS",
    "FHD_NON_XVYCC",
    "FHD_NON_XVYCC_ATMOS",
    "FHD_NON_XVYCC_HDR",
    "FHD_NON_XVYCC_HDR_ATMOS",
    "FHD_XVYCC",
    "FHD_XVYCC_ATMOS",
    "FHD_XVYCC_HDR",
    "FHD_XVYCC_HDR_ATMOS",
};

#endif

#ifndef CC_VGA_EDID_DISABLE
CODE UINT8 VGA_EDID_TABLE[256] =
{
	0x00, 0xFF, 0xFF, 0xFF,  0xFF, 0xFF, 0xFF, 0x00,   0x36, 0x8B, 0x10, 0x00,  0x01, 0x01, 0x01, 0x01,
	0x07, 0x14, 0x01, 0x03,  0x1F, 0x5D, 0x34, 0x78,   0xCA, 0x31, 0xC3, 0xA3,  0x55, 0x4A, 0x9B, 0x24,
	0x10, 0x47, 0x4A, 0xAF,  0xCF, 0x00, 0x01, 0x01,   0x01, 0x01, 0x01, 0x01,  0x01, 0x01, 0x01, 0x01,
	0x01, 0x01, 0x01, 0x01,  0x01, 0x01, 0x1A, 0x36,   0x80, 0xA0, 0x70, 0x38,  0x1F, 0x40, 0x30, 0x20,
	0x35, 0x00, 0xA2, 0x0B,  0x32, 0x00, 0x00, 0x1A,   0x00, 0x00, 0x00, 0xFD,  0x00, 0x32, 0x55, 0x1F,
	0x46, 0x0F, 0x00, 0x0A,  0x20, 0x20, 0x20, 0x20,   0x20, 0x20, 0x00, 0x00,  0x00, 0xFF, 0x00, 0x53,
	0x4B, 0x46, 0x4A, 0x42,  0x4C, 0x30, 0x37, 0x30,   0x32, 0x30, 0x32, 0x39,  0x00, 0x00, 0x00, 0xFC,
	0x00, 0x53, 0x56, 0x34,  0x32, 0x32, 0x58, 0x56,   0x54, 0x0A, 0x20, 0x20,  0x20, 0x20, 0x00, 0x5D,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00
};


//---------------------------------------------------------------------------
// Not loader part
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Static function
//---------------------------------------------------------------------------
void vDrvLoadVgaEdid2Table(void)
{
#ifdef CC_FPGA
    UINT64 u8Offset;
    UINT8 idx = 0;
    //read form eeprom or flash to VGA_EDID_TABLE

#ifdef CC_MULTI_EDID
    UINT32 u4EdidOffset;
    if(DRVCUST_InitQuery(eEdidVgaIndexOffset, &u4EdidOffset) == 0)
    {
        EEPROM_Read((UINT64)u4EdidOffset, (UINT32)&idx, 1);
    }
#endif

    if((idx+1)*VGA_EDID_SIZE > VGA_EDID_STORAGE_SIZE)
    {
        u8Offset = 0;
    }
    else
    {
        u8Offset = idx*VGA_EDID_SIZE;
    }
    //Printf("read edid from eeprom\n");
#ifndef CC_DISABLE_VGA_EDID_FROM_EEP //wait for eeprom ready
    if(EEPVGAEDID_Read(u8Offset,  (UINT32) VGA_EDID_TABLE, VGA_EDID_SIZE ))
    {
        Printf("Read eeprom edid fail\n");
    }
#endif

#endif

}
#endif

#ifndef CC_HDMI_EDID_DISABLE
/*
All value of Port 0 is in HDMIEDID_CEC_192k_DeepColor12B_P0
Arrangement of HDMICEC_SETTING is
Port2 CEC physical address
Port3 CEC physical address
Port4 CEC physical address
*/
UINT8 HDMICEC_SETTING[3]=
{
	0x20, // Port2 CEC physical address
	0x30, // Port2 CEC physical address
	0x40, // Port2 CEC physical address
};

UINT8 HDMIEDID_table[256] =
{
    0x00, 0xFF, 0xFF, 0xFF,  0xFF, 0xFF, 0xFF, 0x00, 0x36, 0x8B, 0x01, 0x00,  0x01, 0x01, 0x01, 0x01,
    0x01, 0x0F, 0x01, 0x03,  0x80, 0x3C, 0x22, 0x78, 0x0A, 0x0D, 0xC9, 0xA0,  0x57, 0x47, 0x98, 0x27,
    0x12, 0x48, 0x4C, 0xBF,  0xEF, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,  0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01,  0x01, 0x01, 0x01, 0x1D, 0x00, 0x72, 0x51, 0xD0,  0x1E, 0x20, 0x6E, 0x28,
    0x55, 0x00, 0xC4, 0x8E,  0x21, 0x00, 0x00, 0x1E, 0x01, 0x1D, 0x80, 0x18,  0x71, 0x1C, 0x16, 0x20,
    0x58, 0x2C, 0x25, 0x00,  0xC4, 0x8E, 0x21, 0x00, 0x00, 0x9E, 0x00, 0x00,  0x00, 0xFC, 0x00, 0x4D,
    0x54, 0x4B, 0x20, 0x4C,  0x43, 0x44, 0x54, 0x56, 0x0A, 0x20, 0x20, 0x20,  0x00, 0x00, 0x00, 0xFD,
    0x00, 0x31, 0x4C, 0x0F,  0x50, 0x0E, 0x00, 0x0A, 0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x01, 0x56,
    0x02, 0x03, 0x23, 0x74,  0x4B, 0x84, 0x10, 0x1F, 0x05, 0x13, 0x14, 0x01,  0x02, 0x11, 0x06, 0x15,
    0x26, 0x09, 0x7F, 0x03,  0x11, 0x7F, 0x18, 0x83, 0x01, 0x00, 0x00, 0x67,  0x03, 0x0C, 0x00, 0x10,
    0x00, 0xB8, 0x2D, 0x01,  0x1D, 0x00, 0xBC, 0x52, 0xD0, 0x1E, 0x20, 0xB8,  0x28, 0x55, 0x40, 0xC4,
    0x8E, 0x21, 0x00, 0x00,  0x1E, 0x01, 0x1D, 0x80, 0xD0, 0x72, 0x1C, 0x16,  0x20, 0x10, 0x2C, 0x25,
    0x80, 0xC4, 0x8E, 0x21,  0x00, 0x00, 0x9E, 0x8C, 0x0A, 0xD0, 0x8A, 0x20,  0xE0, 0x2D, 0x10, 0x10,
    0x3E, 0x96, 0x00, 0x13,  0x8E, 0x21, 0x00, 0x00, 0x18, 0x8C, 0x0A, 0xD0,  0x90, 0x20, 0x40, 0x31,
    0x20, 0x0C, 0x40, 0x55,  0x00, 0x13, 0x8E, 0x21, 0x00, 0x00, 0x18, 0x00,  0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x52
};

BOOL bDrvGetHDMIEdidLddp(void)
{
    INT32 i4Ret;
    UINT64 u8Offset;
    UINT8 u1Lddp;
    u8Offset = (UINT64)DRVCUST_InitGet(eEdidHdmiLddpOffset);

    i4Ret = EEPROM_Read(u8Offset, (UPTR)&u1Lddp, 1);

    if(i4Ret == 0)
    {
        return (BOOL)u1Lddp;
    }

    return 0;
}

UINT8 u1DrvGetHDMIEdidIndex(void)
{
    HDMI_EDID_FLASH_IDX_TYPE eEDIDIdx;
    UINT8 u1PanelEdidIndex, u1FormatIndex = 0;
    PLF_OPT_SEGMENT_T eSegment;
    UINT32 u4Val;
    BOOL bEdidLddp;

    bEdidLddp = bDrvGetHDMIEdidLddp();
    DRVCUST_InitQuery(eOptionSegmentBit,&u4Val);
    eSegment = (PLF_OPT_SEGMENT_T)u4Val;
    EEPHDMIEDID_Format_Read(&u1FormatIndex);

    if((eSegment == PLF_OPT_SEGMENT_KL)||(eSegment == PLF_OPT_SEGMENT_KLW))
    {
        if(u1FormatIndex ==  HDMI_EDID_FORMAT_TYPE_STANDARD)
        {
            eEDIDIdx = (bEdidLddp) ? HDMI_EDID_FHD_XVYCC_ATMOS : HDMI_EDID_FHD_XVYCC;
        }
        else
        {
            eEDIDIdx = (bEdidLddp) ? HDMI_EDID_FHD_XVYCC_HDR_ATMOS : HDMI_EDID_FHD_XVYCC_HDR;
        }
    }
    else if((eSegment == PLF_OPT_SEGMENT_KC) || (eSegment == PLF_OPT_SEGMENT_KC2))
    {

    }
    else
    {
        u1PanelEdidIndex = (UINT8)FlashPanel_GetHdmiEdidIndex();
        if(u1PanelEdidIndex == PANEL_EDID_WXGA_NON_XVYCC)
        {
            if(u1FormatIndex ==  HDMI_EDID_FORMAT_TYPE_STANDARD)
            {
                eEDIDIdx = (bEdidLddp) ? HDMI_EDID_WXGA_NON_XVYCC_ATMOS : HDMI_EDID_WXGA_NON_XVYCC;
            }
            else
            {
                eEDIDIdx = (bEdidLddp) ? HDMI_EDID_WXGA_NON_XVYCC_HDR_ATMOS : HDMI_EDID_WXGA_NON_XVYCC_HDR;
            }
        }
        else
        {
            if(u1FormatIndex ==  HDMI_EDID_FORMAT_TYPE_STANDARD)
            {
                eEDIDIdx = (bEdidLddp) ? HDMI_EDID_FHD_NON_XVYCC_ATMOS : HDMI_EDID_FHD_NON_XVYCC;
            }
            else
            {
                eEDIDIdx = (bEdidLddp) ? HDMI_EDID_FHD_NON_XVYCC_HDR_ATMOS : HDMI_EDID_FHD_NON_XVYCC_HDR;
            }
        }
    }

    return eEDIDIdx;
}

void vDrvLoadHdmiEdid2Table(void)
{
#ifndef CC_FPGA
    UINT64 u8Offset;
    UINT8 idx = 0;
    //read form eeprom or flash to VGA_EDID_TABLE

    idx = u1DrvGetHDMIEdidIndex();

    u8Offset = idx*HDMI_EDID_SIZE;

    if (EEPHDMIEDID_Read(u8Offset,  (UINT32) HDMIEDID_table, HDMI_EDID_SIZE ))
    {
        Printf("[HDMI Edid]Read eeprom edid fail\n");
    }

#endif
}
#endif

//---------------------------------------------------------------------------
// Not loader part
//---------------------------------------------------------------------------
#ifndef CC_MTK_LOADER
//---------------------------------------------------------------------------
// Static function
//---------------------------------------------------------------------------
INT32 i4DrvSetHDMIEdidLddp(UINT8 u1Lddp)
{
    UINT64 u8Offset;

    u8Offset = (UINT64)DRVCUST_InitGet(eEdidHdmiLddpOffset);

    return EEPROM_Write(u8Offset, (UPTR)&u1Lddp, 1);
}

//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------
void vHDMIEdidSetDDP(UINT8 u1DDPEnable)
{
    UINT8 u1HdmiEdidLDDP;
    E_HDMI_SWITCH_NUM eHdmiPort = HDMI_SWITCH_1;

    u1HdmiEdidLDDP = (UINT8)bDrvGetHDMIEdidLddp();
    Printf("[HDMI EDID]Set DDP:%d, LDDP:%d!\n",u1DDPEnable, u1HdmiEdidLDDP);

    if(u1HdmiEdidLDDP != u1DDPEnable)
    {
        u1HdmiEdidLDDP = u1DDPEnable;

        if(u1HdmiEdidLDDP > 0)
        {
            i4DrvSetHDMIEdidLddp(1);
        }
        else
        {
            i4DrvSetHDMIEdidLddp(0);
        }

        vHDMI_INTERNAL_EDID_Init();

        for(eHdmiPort = HDMI_SWITCH_1; eHdmiPort < HDMI_SWITCH_MAX; eHdmiPort++)
        {
            vSetHdmiStateEx(eHdmiPort, HDMI_STATE_INIT);
        }
    }
}

//---------------------------------------------------------------------------
// Cli cmd
//---------------------------------------------------------------------------
static INT32 _SifHDMISetEdidDdpCmd(INT32 i4Argc, const CHAR **szArgv)
{
    UINT8 u1Lddp = 0;

    if(i4Argc != 2)
    {
        u1Lddp = (UINT8)bDrvGetHDMIEdidLddp();
        Printf("Usage: <Normal(0)/Atmos(1)>\n");
        Printf("Edid LDDP:%d\n", u1Lddp);
        return (0);
    }

    u1Lddp = (UINT8) StrToInt(szArgv[1]);

    vHDMIEdidSetDDP(u1Lddp);

    return 0;
}

static INT32 _SifHDMIQuerySonyEdidCmd(INT32 i4Argc, const CHAR **szArgv)
{
    HDMI_EDID_FLASH_IDX_TYPE eEDIDIdx;
    UINT8 u1PanelEdidIndex, u1FormatIndex = 0;
    PLF_OPT_SEGMENT_T eSegment;
    UINT32 u4Val;
    BOOL bEdidLddp;
    UINT8 u1ATMOSService;

    bEdidLddp = bDrvGetHDMIEdidLddp();
    EEPHDMIEDID_Format_Read(&u1FormatIndex);
    DRVCUST_InitQuery(eOptionSegmentBit,&u4Val);
    u1PanelEdidIndex = (UINT8)FlashPanel_GetHdmiEdidIndex();
    eSegment = (PLF_OPT_SEGMENT_T)u4Val;

    Printf("=========Sony EDID information=========\n");
    Printf("HDMI Edid index:%d\n", u1PanelEdidIndex);
    Printf("LDDP:%d\n", bEdidLddp);
    Printf("Segment:%s\n", _aszSonySegment[eSegment]);

    if(u1FormatIndex == HDMI_EDID_FORMAT_TYPE_STANDARD)
    {
        Printf("Format:Standard\n");
    }
    else
    {
        Printf("Format:Enhance\n");
    }

    //ATMOS_CEC_DISABLE Service Register in EEPROM 0x30A0
    EEPROM_Read(0x30A0, (UPTR)&u1ATMOSService, 1);
    if(u1ATMOSService == 0xA)
    {
        Printf("ATMOS:Normal Model\n");
    }
    else
    {
        Printf("ATMOS:Service Model\n");
    }

    eEDIDIdx = u1DrvGetHDMIEdidIndex();
    Printf("Select EDID:%s\n", _aszSonyEdidName[eEDIDIdx]);

    return 0;
}

#endif
#endif // _EDID_CUSTOMIZATION_H_
