/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*----------------------------------------------------------------------------*/
/*! @file 3d_photo_playback.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/11/26 $
 *  $Author: dtvbm11 $
 *  
 *  @par Description:
 *         This file exports all image library API and constant for 
 *         other module using.
 */
/*----------------------------------------------------------------------------*/

#ifndef _MPO_PHOTO_PLAYBACK_H_
#define _MPO_PHOTO_PLAYBACK_H_


#include "x_gl.h"
#include "x_img.h"
#include "x_wgl.h"
#include "x_dbg.h"
#include "x_img_tfx.h"


#define IMG_RC_DISPLAY_NAME                 SN_PRES_MAIN_DISPLAY

#define _IMG_DRC_DISPLAY_RES_MAX_COUNT       ((UINT8) 4)

#define _IMG_DRC_FLAG_READY               ((UINT8) 0x01)
#define _IMG_DRC_FLAG_USED                   ((UINT8) 0x02)

#define _IMG_PE_ACCURACY_CONST              ((UINT32)10000)


typedef struct _IMG_CROP_RECT_T
{
    UINT32  ui4_left_offset;
    UINT32  ui4_right_offset;
    UINT32  ui4_top_offset;
    UINT32  ui4_bottom_offset;
}IMG_CROP_RECT_T;


typedef struct _IMG_DRC_BUF_T
{
    GL_HVIEW_T                  h_view;                         /*view handle*/
    GL_HSURFACE_T               h_surf;                         /*surface handle*/
    IMG_ROTATE_OP_TYPE_T        e_rotate;                       /*rotation of this display*/
    GL_POINT_T                  t_center_offset;
    UINT32                      ui4_scale;                       /*scale of this display*/
    UINT32                      ui4_x;
    UINT32                      ui4_y;
    UINT32                      ui4_w;
    UINT32                      ui4_h;
    UINT32                      ui4_buf_width;
    UINT32                      ui4_buf_height;

    IMG_CROP_RECT_T             t_surface_crop_rect;
}   IMG_DRC_BUF_T;

typedef struct _IMG_DRC_DISPLAY_RES_T
{
    IMG_DRC_BUF_T               t_drc_buf;
    UINT32                      ui4_viewport;     /*viewport of this display*/
    UINT8                       ui1_flag;
}   IMG_DRC_DISPLAY_RES_T;

typedef struct _IMG_METADATA_T
{
    UINT32                      ui4_src_width;
    UINT32                      ui4_src_height;
}IMG_METADATA_T;


typedef struct
{
    HANDLE_T                    h_img;
    IMG_DRC_BUF_T*              pt_buf_crnt_decode;
    IMG_DRC_BUF_T*              pt_buf_crnt_rotate;
    IMG_DRC_BUF_T*              pt_buf_animation_gif;
}IMG_FILE_BUF_T;


#define FILE_CURRENT_IDX    0
#define FILE_NEXT_IDX       1
#define FILE_TOTAL_IDX      2

#define SVCTX_NAME_MAIN     "main_svctx"


typedef struct _IMG_DRC_DATA_T
{
    GL_HSCREEN_T                h_screen;
    GL_HPLANE_T                 h_plane;
    UINT32                      ui4_screen_w;
    UINT32                      ui4_screen_h;
    UINT32                      ui4_canvas_w;
    UINT32                      ui4_canvas_h;
    INT8                        i1_img_z_order;
    UINT32                      ui4_flag;
    GL_POINT_T                  t_screen_offset;
    IMG_DRC_DISPLAY_RES_T      at_disp_pool[_IMG_DRC_DISPLAY_RES_MAX_COUNT];
    IMG_DRC_BUF_T*              pt_buf_crnt_disp;
    IMG_FILE_BUF_T              at_file_info[FILE_TOTAL_IDX];

    BOOL                        b_init;

    IMG_METADATA_T              t_image_metadata;

    HANDLE_T                    h_scc_main_vid;
    BOOL                        b_enable_3d;
    HANDLE_T                    h_svctx;

    HANDLE_T                    h_tfx;
    INT32                       i4_status;
    HANDLE_T                    h_tfx_lock;
    BOOL                        b_effect;
    HANDLE_T                    h_mutex_lock;

    UINT8                       ui1_crnt_img_idx;
    x_img_event_nfy_fct         pf_img_nfy;
    VOID*                       pv_img_tag;
}   IMG_DRC_DATA_T;

typedef struct
{
    INT32 i4_status;
    VOID* pv_data;
}IMG_TFX_MSG_T;

#define TYPE_OFFSET 100
typedef enum
{
    IMG_NFY_DEC_HANDLE_CLOSED_TYPE = TYPE_OFFSET,
    IMG_NFY_DEC_LOAD_COMPLETE_TYPE,
    IMG_NFY_DEC_LOAD_COMPLETE_WITH_TAG,
    IMG_NFY_DEC_GIF_ONE_FRM_LOAD_COMPLETE_TYPE,
    IMG_NTY_DEC_GIF_LOAD_STOP
} IMG_DEC_NFY_EVENT_TYPE_T;


extern INT32 x_img_playback_rc_init(VOID);
extern INT32 x_img_playback_rc_deinit(VOID);
extern INT32 x_img_playback_open(const CHAR* ps_file_name, x_img_nfy_fct   pf_nfy, const VOID* pv_tag, HANDLE_T* ph_img);
extern INT32 x_img_playback_decode(HANDLE_T h_img);
extern INT32 x_img_playback_display(HANDLE_T h_img);
extern INT32 x_img_playback_close(HANDLE_T h_img);
extern INT32 x_img_playback_get(HANDLE_T h_img, IMG_INFO_TYPE_T e_info_type, VOID* pv_info, UINT32* pui4_len);
extern INT32 x_img_playback_set_3d(HANDLE_T h_img, BOOL b_enable_3d);
extern INT32 x_img_playback_rotate(HANDLE_T h_img, IMG_ROTATE_OP_TYPE_T e_rotate_type);
extern INT32 x_img_playback_tfx_play(HANDLE_T h_img, IMG_TFX_EFFECT_T    e_effect);
extern INT32 x_img_playback_tfx_stop(VOID);
extern INT32 x_img_playback_get_thumbnail(
    const CHAR*         ps_file_name,
    UINT32              ui4_width, 
    UINT32              ui4_height, 
    IMG_COLORMODE_T     e_cm,
    VOID*               pv_addr);

extern INT32 x_img_playback_reg_callback(x_img_event_nfy_fct  pf_nfy, VOID * pv_tag);

#endif

